; level meter with 4 stereo inputs
; designed for ATmega8535 and wintek2402
;*****************
.include "m8535def.inc"
;*****************
; programm compiler's options
.equ WH24		= 1; 1 - WH2402, 0 - WH1602
;*****************
; constants
; in-out port constants
; PORTA - analog inputs
; PORTB - PRG and Sensor
; PORTC - LCD data bus
; PORTD - LCD controls, keys and outs
.equ LCDCMDPORT	= PORTD
.equ SENSOR_PORT	= PINB
.equ LCDDATAPORT	= PORTC
.equ INKEYPORT	= PIND
.equ OUT_LED_PORT	= PORTD
;*****************
; PORTA pins
.equ AINL_1	= PORTA0
.equ AINR_1	= PORTA1
.equ AINL_2	= PORTA2
.equ AINR_2	= PORTA3
.equ AINL_3	= PORTA4
.equ AINR_3	= PORTA5
.equ AINL_4	= PORTA6
.equ AINR_4	= PORTA7
; PORTB pins
.equ SENSOR	= PINB0
; PORTD pins
.equ KEYIN_0	= PORTD0
.equ KEYIN_1	= PORTD1
.equ KEYIN_2	= PORTD2
.equ KEYIN_3	= PORTD3
.equ KEYIN_4	= PORTD4
.equ LEDOUT	= PORTD5
.equ EOUT		= PORTD7
.equ RSOUT		= PORTD6
;-----------------------
; KEY bit codes
.equ KEY_UP	= 0
.equ KEY_DOWN	= 1
.equ KEY_ENTER	= 2
.equ KEY_RET	= 3
;***********************
; time constants
.equ LCDREFRESH_TIME	= 50; ms
.equ SHOWSPLASH_TIME	= 1000; ms
.equ LCDFLASH_PERIOD	= 1000; ms
.equ KEYCONTROL_TIME	= LCDREFRESH_TIME
.equ ANTIDREBEZG_DELAY	= 100/KEYCONTROL_TIME;100 ms
.equ KEYREPEAT_DELAY	= 1000/KEYCONTROL_TIME;1000 ms
.equ AUTOREPEAT_DELAY	= 200/KEYCONTROL_TIME;200 ms
.equ AUTOREPEAT_STEP	= 10/KEYCONTROL_TIME;10 ms
.equ LEVELCONTROL_PERIOD	= 500; ms
.equ REQUEST_FR = 400; Hz
;-----------------------
.equ QUARTZ		= 8000000; quartz frequency, Hz
.equ TMRPRESCALER0	= 256;8;0-1-8-64-256-1024
.equ CNT0_RELOAD	= QUARTZ / TMRPRESCALER0 / REQUEST_FR / 2- 1; 400 Hz buzzer
.equ TMRPRESCALER1	= 256;0-1-8-64-256-1024
;.equ CNT_TIMERCONTROL	= 500 * QUARTZ / TMRPRESCALER1 / 1000 - 1; 500ms
.equ TMRPRESCALER2	= 8;0-1-8-64-256-1024
.equ CNT2_RELOAD	= QUARTZ / TMRPRESCALER2 / 8 / 8000 - 1; 8000 kHz sample rate, 8 ch
.equ CNT_LCDREFRESH	= LCDREFRESH_TIME * QUARTZ / TMRPRESCALER1 / 1000 - 1
.equ ADCPRESCALER	= 4;2-4-8-32-64-128
;***********************
; LCD constants
.equ LCD_NOTCH_CODE		= 8; symbol displaying level no left and right bar
.equ LCD_LEFTCH_CODE	= 9; symbol displaying level left bar
.equ LCD_RIGHTCH_CODE	= 10; symbol displaying level right bar
.equ LCD_LEFTRIGHTCH_CODE	= 11; symbol displaying level left and right bar
;********************
; check constants
.IF (AUTOREPEAT_DELAY - AUTOREPEAT_STEP) <= 0
.ERROR "no correct autorepeat key data!"
.ENDIF
.IF ANTIDREBEZG_DELAY <= 0
.ERROR "no correct key's antidrebezg delay!"
.ENDIF
.IF LEVELCONTROL_PERIOD / LCDREFRESH_TIME * LCDREFRESH_TIME != LEVELCONTROL_PERIOD
.ERROR "not correct level control period!"
.ENDIF
.IF (LCDREFRESH_TIME < 50) || (1000 / LCDREFRESH_TIME * LCDREFRESH_TIME != 1000)
.ERROR "refresh time not correct!"
.ENDIF
;*********************
; variables
.def level0	= r0
.def level1	= r1
.def level2	= r2
.def level3	= r3
.def isign	= r4
.def keys_code	= r5
.def key_timer	= r6
.def ld_timer	= r7
.def old_key	= r8
.def repeat_delay	= r9
.def flash_timer	= r10
;.def relay_timer	= r11
.def st_store1	= r12
.def st_store2	= r13
.def eeprom_data	= r14
.def zero	= r15

.def temp	= r16
.def tmp1	= r17
.def tmp2	= r18
.def adc_channel	= r19
.def mode	= r20
.def itemp	= r21
.def scrmode	= r22
.def barcode	= r23
.def itempL	= r24
.def itempH	= r25
; xh.xl average level calculate	= r26, r27
; yh.yl	lcd refresh	= r28, r29
; zh.zl lcd	ram buf	= r30, r31
;********************
; RAM memory allocate
.dseg
.IF WH24
.equ LCD_STR_LENGTH	= 24
  .ELSE
.equ LCD_STR_LENGTH	= 16
.ENDIF
.equ QUANTITY_CHANNEL = 8
LCDbuff:
LCDbuffSTR1:	.BYTE	LCD_STR_LENGTH
LCDbuffSTR2:	.BYTE	LCD_STR_LENGTH
;
.equ LVLTBL_SIZE	= 8
RLEVEL_TABLE:	.BYTE	QUANTITY_CHANNEL * LVLTBL_SIZE
;-------- bytes for each channel
; current level	2 bytes
; part of average level	2 bytes
; average level	2 bytes
.equ AVERAGE_SHIFT	= 4
; reserved		2 bytes
;--------
RGAININ_1:	.BYTE	1
RGAININ_2:	.BYTE	1
RLEVELSCALE_1:	.BYTE	2
RLEVELSCALE_2:	.BYTE	2
RCURSOR_POSITION:
RSPLASH_TIME:	.BYTE	1
;********************
; EEPROM flash memory allocate
.eseg
FM_GAININ_1:	.BYTE	1
FM_GAININ_2:	.BYTE	1
FM_MODE:		.BYTE	1
;-----------------
; check memory allocate
.IF high(RLEVELSCALE_2 + 1) != high(RLEVELSCALE_1 + 1)
.ERROR "change memory address for LevelScale!"
.ENDIF
;*****************
.cseg
.org 0
	rjmp RESET ; Reset Handler
	reti;rjmp EXT_INT0 ; IRQ0 Handler
	reti;rjmp EXT_INT1 ; IRQ1 Handler
	rjmp TIM2_COMP ; ADC sample rate Timer2 Compare Handler
	reti;rjmp TIM2_OVF ; Timer2 Overflow Handler
	reti;rjmp TIM1_CAPT ; Timer1 Capture Handler
	reti;rjmp TIM1_COMPA ; Timer1 Compare A Handler
	rjmp TIM1_COMPB ; 0,05 s Timer1 Compare B Handler
	reti;rjmp TIM1_OVF ; Timer1 Overflow Handler
	reti;rjmp TIM0_OVF ; Timer0 Overflow Handler
	reti;rjmp SPI_STC ; SPI Transfer Complete Handler
	reti;rjmp USART_RXC ; USART RX Complete Handler
	reti;rjmp USART_UDRE ; UDR Empty Handler
	reti;rjmp USART_TXC ; USART TX Complete Handler
	reti;rjmp ADC_INT ; ADC Conversion Complete Handler
	reti;rjmp EE_RDY ; EEPROM Ready Handler
	reti;rjmp ANA_COMP ; Analog Comparator Handler
	reti;rjmp TWSI ; Two-wire Serial Interface Handler
	reti;rjmp EXT_INT2 ; IRQ2 Handler
	reti;rjmp TIM0_COMP ; 400 Hz Timer0 Compare Handler
	reti;rjmp SPM_RDY ; Store Program Memory Ready Handler
;****************
; initialisation
RESET:
	cli
	ldi	temp, low(RAMEND)
	out	spl, temp
	ldi	temp, high(RAMEND)
	out	sph, temp
;----------------
	wdr
; set modes
	ldi	temp, ((0 << WDE) | (0 << WDCE) | (1 << WDP2) | (1 << WDP1) | (0 << WDP0))
	out	WDTCR, temp
; watch dog timer initialization 1 s
;	ldi	temp, (1 << WDCE) | (1 << WDE)
;	out	WDTCSR, temp
;	ldi	temp, (1 << WDE) | (0 << WDCE) | (1 << WDP2) | (1 << WDP1) | (0 << WDP0)
;	out	WDTCSR, temp
;----------------
	clr	zero
	out	MCUSR, zero
; sleep mode=idle
	ldi	temp, (0 << SM2)|(0 << SE)|(0 << SM1)|(0 << SM0)|(1 << ISC11)|(1 << ISC10)|(1 << ISC01)|(1 << ISC00)
	out	MCUCR, temp
;----------------
; set default port states, PUD in MCUSR must be 0
	out	SFIOR, zero
	out PORTA, zero
	ldi	temp, ~(1<<SENSOR)
	out	PORTB, temp
	out	PORTC, zero
	ldi	temp, (0<<EOUT)|(0<<RSOUT)|(0<<LEDOUT)|(1<<KEYIN_0)|(1<<KEYIN_1)|(1<<KEYIN_2)|(1<<KEYIN_3)|(1<<KEYIN_4)
	out	PORTD, temp
; port directions
; DDR 1 - output, PORT must be 0
; DDR 0 - input, if PORT=0 - tristate
	out	DDRA, zero
	ser	temp
	out	DDRC, temp
	ldi	temp, (1 << EOUT) | (1 << RSOUT) | (1 << LEDOUT)
	out	DDRD, temp
	out	DDRB, zero
;---------------
	rcall	LCDinit
	ldi	zl, low(2 * CharLevels)
	ldi	zh, high(2 * CharLevels)
	rcall WriteCharacters
	ldi	zl, low(2 * ScrShowID)
	ldi	zh, high(2 * ScrShowID)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	MemToLCD
;---------------
; init data
	out		EEARH, zero
	rcall	Init
;----------------
; set timer settings for measuring CTC(1<<WGM12) normal(0<<WGM12)
.IF TMRPRESCALER1 == 1024
.equ	CSBITS1 = (1<<CS12)|(0<<CS11)|(1<<CS10); timer prescaler clk/1024
.ELSE
.IF TMRPRESCALER1 == 256
.equ	CSBITS1 = (1<<CS12)|(0<<CS11)|(0<<CS10); timer prescaler clk/256
.ELSE
.IF TMRPRESCALER1 == 64
.equ	CSBITS1 = (0<<CS12)|(1<<CS11)|(1<<CS10); timer prescaler clk/64
.ELSE
.IF TMRPRESCALER1 == 8
.equ	CSBITS1 = (0<<CS12)|(1<<CS11)|(0<<CS10); timer prescaler clk/8
.ELSE
.IF TMRPRESCALER1 == 1
.equ	CSBITS1 = (0<<CS12)|(0<<CS11)|(1<<CS10); timer prescaler clk/1
.ELSE
.IF TMRPRESCALER1 == 0
.equ	CSBITS1 = (0<<CS12)|(0<<CS11)|(0<<CS10); timer prescaler no clk
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
; on CTC mode, set data
	ldi	temp, (0<<ICNC1)|(0<<ICES1)|(0<<WGM13)|(0<<WGM12)|CSBITS1
	out	TCCR1B, temp
;-----------------
; timer1 init 0,5 s period
	out	TCCR1A, zero
;	out	TCNT1H, zero
;	out	TCNT1L, zero
;	ldi	temp, high(CNT_BUZZER);(CNT_TIMERCONTROL)
;	out	OCR1AH, temp
;	ldi	temp, low(CNT_BUZZER);(CNT_TIMERCONTROL)
;	out	OCR1AL, temp
; LCD refresh and KEYS control
	ldi	temp, high(CNT_LCDREFRESH)
	out	OCR1BH, temp
	ldi	temp, low(CNT_LCDREFRESH)
	out	OCR1BL, temp
;----------------
; set timer0 settings for measuring ADC CTC(1<<WGM01) normal(0<<WGM01)
.IF TMRPRESCALER0 == 1024
.equ	CSBITS0 = (1<<CS02)|(0<<CS01)|(1<<CS00); timer prescaler clk/1024
.ELSE
.IF TMRPRESCALER0 == 256
.equ	CSBITS0 = (1<<CS02)|(0<<CS01)|(0<<CS00); timer prescaler clk/256
.ELSE
.IF TMRPRESCALER0 == 64
.equ	CSBITS0 = (0<<CS02)|(1<<CS01)|(1<<CS00); timer prescaler clk/64
.ELSE
.IF TMRPRESCALER0 == 8
.equ	CSBITS0 = (0<<CS02)|(1<<CS01)|(0<<CS00); timer prescaler clk/8
.ELSE
.IF TMRPRESCALER0 == 1
.equ	CSBITS0 = (0<<CS02)|(0<<CS01)|(1<<CS00); timer prescaler clk/1
.ELSE
.IF TMRPRESCALER0 == 0
.equ	CSBITS0 = (0<<CS02)|(0<<CS01)|(0<<CS00); timer prescaler no clk
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
; on CTC mode, set data
	ldi	temp, (0<<FOC0)|(0<<WGM00)|(0<<COM01)|(0<<COM00)|(1<<WGM01)|CSBITS0
	out	TCCR0, temp
	ldi	temp, CNT0_RELOAD
	out	OCR0, temp
;----------------
; set timer2 settings for measuring ADC CTC(1<<WGM21) normal(0<<WGM21)
.IF TMRPRESCALER2 == 1024
.equ	CSBITS2 = (1<<CS22)|(0<<CS21)|(1<<CS20); timer prescaler clk/1024
.ELSE
.IF TMRPRESCALER2 == 256
.equ	CSBITS2 = (1<<CS22)|(0<<CS21)|(0<<CS20); timer prescaler clk/256
.ELSE
.IF TMRPRESCALER2 == 64
.equ	CSBITS2 = (0<<CS22)|(1<<CS21)|(1<<CS20); timer prescaler clk/64
.ELSE
.IF TMRPRESCALER2 == 8
.equ	CSBITS2 = (0<<CS22)|(1<<CS21)|(0<<CS20); timer prescaler clk/8
.ELSE
.IF TMRPRESCALER2 == 1
.equ	CSBITS2 = (0<<CS22)|(0<<CS21)|(1<<CS20); timer prescaler clk/1
.ELSE
.IF TMRPRESCALER2 == 0
.equ	CSBITS2 = (0<<CS22)|(0<<CS21)|(0<<CS20); timer prescaler no clk
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
; on CTC mode, set data
	ldi	temp, (0<<FOC2)|(0<<WGM20)|(0<<COM21)|(0<<COM20)|(1<<WGM21)|CSBITS2
	out	TCCR2, temp
	ldi	temp, CNT2_RELOAD
	out	OCR2, temp
	out	ASSR, zero
; timer interrupt configure
; compare match A and B interrupt enable, overflow interrupt enable
	ldi	temp, (1 << OCIE2)|(0 << TOIE2)|(0 << TICIE1)|(0 << OCIE1A)|(1 << OCIE1B)|(0 << TOIE1)|(0 << OCIE0)|(0 << TOIE0)
	out	TIMSK, temp
;---------------
; ADC initialization, internal 2.56 v
	sbi	ACSR, ACD;comparator disable;out	ACSR, zero; comparator enable
	ldi	temp, (1<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX4)|(0<<MUX3)|(0<<MUX2)|(0<<MUX1)|(0<<MUX0)
	out	ADMUX, temp
;	cbi	SFIOR, ADTS2; free running mode
;	cbi	SFIOR, ADTS1
;	cbi	SFIOR, ADTS0
	in	temp, SFIOR
	ori	temp, (0<<ADTS2)|(0<<ADTS1)|(0<<ADTS0);(0<<ADTS2)|(1<<ADTS1)|(1<<ADTS0); timer/counter0 compare match
	out	SFIOR, temp
.IF ADCPRESCALER == 128
.equ	ADPSBITS = (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
.ELSE
.IF ADCPRESCALER == 64
.equ	ADPSBITS = (1<<ADPS2)|(1<<ADPS1)|(0<<ADPS0)
.ELSE
.IF ADCPRESCALER == 32
.equ	ADPSBITS = (1<<ADPS2)|(0<<ADPS1)|(1<<ADPS0)
.ELSE
.IF ADCPRESCALER == 16
.equ	ADPSBITS = (1<<ADPS2)|(0<<ADPS1)|(0<<ADPS0)
.ELSE
.IF ADCPRESCALER == 8
.equ	ADPSBITS = (0<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
.ELSE
.IF ADCPRESCALER == 4
.equ	ADPSBITS = (0<<ADPS2)|(1<<ADPS1)|(0<<ADPS0)
.ELSE
.IF ADCPRESCALER == 2
.equ	ADPSBITS = (0<<ADPS2)|(0<<ADPS1)|(1<<ADPS0)
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
	ldi	temp, (1<<ADEN)|(1<<ADSC)|(0<<ADATE)|(1<<ADIF)|(0<<ADIE)|ADPSBITS
	out	ADCSRA, temp
;---------------
; external interrupt configure for pulse and power control
;	ldi	temp, (1 << INT1) | (1 << INT0) | (0 << PCIE)
;	out	GIMSK, temp
;	out	EIFR, temp
;---------------
; main
	ldi	scrmode, MSHOWLEVELS
	clr	keys_code
	clr	key_timer
	ser	temp
	mov	old_key, temp
;	clr	repeat_delay
	clr	ld_timer
	ldi	temp, LCDFLASH_PERIOD / LCDREFRESH_TIME
	mov	flash_timer, temp
	ldi	temp, SHOWSPLASH_TIME / LCDREFRESH_TIME
	sts	RSPLASH_TIME,temp
	sei
MAIN:
	wdr
	ldi	temp, (0<<SM2)|(1<<SE)|(0<<SM1)|(0<<SM0)|(0<<ISC11)|(0<<ISC10)|(0<<ISC01)|(0<<ISC00)
	out	MCUCR, temp
	sleep
	rjmp	MAIN
;****************
; save device state
SaveState:
	ldi	temp, FM_MODE
	mov	eeprom_data, mode
	rcall	EEPROM_write
	ret
;****************
; ADC 4 ch, 32000 Hz, 8000 Hz sample rate
TIM2_COMP:
;ADC_INT:
; X, level0...level3, itempL, itempH
	in	st_store1, SREG
;	push	temp
; calculate rms
; Y = Y + (X - Y) * K; K = dT / (t + dT);
; load level for channel from memory
	in	itempL, ADCL
	in	itempH, ADCH
	in	itemp, ADMUX
;
	inc	itemp
	andi	itemp, 0xe7
	out	ADMUX, itemp
	dec	itemp
;
	andi	itemp, 0x07
; multiply
;	ldi	xl, low(RLEVELSCALE_1)
;	ldi	xh, high(RLEVELSCALE_1)
;	sbrc	itemp, 1
;	ldi	xl, low(RLEVELSCALE_2)
;	ld	r4, X+
;	mul	r4, itempH
;	movw	r3:r2, r1:r0
;	mul r4, itempL
;	add	r2, r1
;	adc	r3, zero
;	ld	r4, X
;	mul	r4, itempL
;	add	r2, r0
;	adc	r3, r1
;	mul	r4, itempH
;	add	r3, r0
;	mov	itempL, r2
;	mov	itempH, r3
;
	ldi	xl, low(RLEVEL_TABLE)
	ldi	xh, high(RLEVEL_TABLE)
	lsl	itemp
	lsl	itemp
	lsl	itemp
	add	xl, itemp
	adc	xh, zero
	st	x+, itempL; store level
	st	x+, itempH
	ld	level0, x+
	ld	level1, x+
	ld	level2, x+
	ld	level3, x
;---- X - Y
	sub	itempL, level2
	sbc	itempH, level3
	sbc	isign, isign
;---- * K, /2048, >> 11
	clr	itemp
	asr	itempH
	ror	itempL; /512
	ror	itemp; /512
	asr	itempH
	ror	itempL; /1024
	ror	itemp; /1024
	asr	itempH
	ror	itempL; /2048
	ror	itemp; /2048
;---- + Y
	add	level0, itemp
	adc	level1, itempL
	adc	level2, itempH
	adc	level3, isign
;----------------
	st	x, level3
	st	-x, level2
	st	-x, level1
	st	-x, level0
;----------------
	sbi	ADCSRA, ADSC
;	pop	temp
	out	SREG, st_store1
	reti
;****************
; timer request
;TIM1_COMPA:
TIM0_COMP:
	reti
;****************
; address table for modes
.equ MSHOWID	= 0
.equ MSHOWLEVELS	= 1
.equ MSHOWLEVELCH1	= 2
.equ MSHOWLEVELCH2	= 3
.equ MSHOWLEVELCH3	= 4
.equ MSHOWLEVELCH4	= 5
DTmodes:
.dw	ShowID, ShowLevels, ShowLevelCh1, ShowLevelCh2, ShowLevelCh3, ShowLevelCh4
;****************
; LCD refresh, KEYS control, 20 Hz, 0.05 s
TIM1_COMPB:
	sei
	in	st_store2, SREG
	push	temp
;
	in	zl, TCNT1L
	in	zh, TCNT1H
	ldi	temp, low(CNT_LCDREFRESH)
	add	zl, temp
	ldi	temp, high(CNT_LCDREFRESH)
	adc	zh, temp
	out	OCR1BH, zh
	out	OCR1BL, zl
;
	rcall	KeyControl
;--- show id splash 3 sec
ShowID:
	lds	temp, RSPLASH_TIME
	tst	temp
	breq	Sensor_check
	dec temp
	sts	RSPLASH_TIME, temp
	pop	temp
	out	SREG, st_store2
	reti
;--
Sensor_check:
	sbic	SENSOR_PORT, SENSOR
	rjmp	Sensor_OK
	cbi	OUT_LED_PORT, LEDOUT
	rcall	ShowSensorUnplugged
	pop	temp
	out	SREG, st_store2
	reti
;--
Sensor_OK:
	sbi	OUT_LED_PORT, LEDOUT
	rcall	SelectScrMode
	pop	temp
	out	SREG, st_store2
	reti
;***************
; read keys' state and send message for program
KeyControl:
;	tst	keys_code
;	brne	KeyControlRet; if keys_code != 0, key code not computed or not present
	cpse	keys_code, zero
	ret
;--
	cp	zero, key_timer
	sbc	key_timer, zero
	in	temp, INKEYPORT
; antidrebezg and repeat
	ori	temp, ~((1<<KEYIN_4) | (1<<KEYIN_3) | (1<<KEYIN_2) | (1<<KEYIN_1) | (1<<KEYIN_0))
	cpi	temp, 0xff
	breq	KCNoKey
	cp	old_key, temp
	breq	KCTimerCheck
	mov	old_key, temp
	ldi	tmp1, ANTIDREBEZG_DELAY
	mov	key_timer, tmp1
	ldi	tmp1, KEYREPEAT_DELAY
	mov	repeat_delay, tmp1
KCTimerCheck:
	tst	key_timer
	brne	KCNoCode
; auto repeat for up and down keys
	sbrs	temp, KEYIN_0;KEY_UP
	rjmp	KCAutoRepeatEnable
	sbrc	temp, KEYIN_1;KEY_DOWN
	rjmp	KCNoRepeatKey;KCGetKeyCode
KCAutoRepeatEnable:
	mov	key_timer, repeat_delay
	ldi	tmp1, AUTOREPEAT_DELAY
	cp	tmp1, repeat_delay
	brcc	KCDecStep
	mov	repeat_delay, tmp1
KCDecStep:
	ldi	tmp1, AUTOREPEAT_STEP
	sub	repeat_delay, tmp1
	brcc	KCGetKeyCode
	add	repeat_delay, tmp1
KCGetKeyCode:
; if pressed more than one key, error
	sbrs	temp, KEYIN_0;KEY_UP
	inc	keys_code
	sbrs	temp, KEYIN_1;KEY_DOWN
	inc	keys_code
	sbrs	temp, KEYIN_2;KEY_ENTER
	inc	keys_code
	sbrs	temp, KEYIN_3;KEY_RET
	inc	keys_code
;	sbrs	temp, KEYIN_2;KEYIN_4;KEY_START
;	inc	keys_code
;	sbrs	temp, KEYIN_3;KEYIN_5;KEY_STOP
;	inc	keys_code
	sbrs	temp, KEYIN_4;KEY_BUZZEROFF_AUTO
	inc	keys_code
	dec	keys_code
	mov	keys_code, zero
	brne	KeyControlRet
;- change pin code to key code
	mov	keys_code, temp
	com	keys_code
KCNoCode:
KeyControlRet:
	ret
;--
KCNoKey:
	mov	old_key, temp
	ret
;--
KCNoRepeatKey:
	ldi	tmp1, KEYREPEAT_DELAY
	cp	repeat_delay, tmp1
	mov	repeat_delay, zero
	breq	KCGetKeyCode
	ret
;***************
;show message "sensor unplugged"
ShowSensorUnplugged:
	ldi	zl, low(2 * ScrSensorUnplugged)
	ldi	zh, high(2 * ScrSensorUnplugged)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	MemToLCD
	ret
;***************
;--- select screen modes
SelectScrMode:
	ldi	zl, low(2 * DTmodes)
	ldi	zh, high(2 * DTmodes)
	add	zl, scrmode
	adc	zh, zero
	add	zl, scrmode
	adc	zh, zero
	lpm	temp, z+
	lpm	zh, z
	mov zl, temp
	ijmp
;--- show 4 ch stereo level indicator
ShowLevels:
	ldi	zl, low(2 * ScrShowLevels)
	ldi	zh, high(2 * ScrShowLevels)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	DrawBarCh1
	rcall	DrawBarCh2
	rcall	DrawBarCh3
	rcall	DrawBarCh4
	rcall	MemToLCD
	mov	temp, keys_code
	clr	keys_code
	sbrs	temp, KEY_ENTER
	ret
;--
	ldi	zl, low(2 * Characters)
	ldi	zh, high(2 * Characters)
	rcall WriteCharacters
	ldi	scrmode, MSHOWLEVELCH1
	ret
;--- show 1-st ch stereo level indicator
ShowLevelCh1:
	ldi	zl, low(2 * ScrShowLevelCh1)
	ldi	zh, high(2 * ScrShowLevelCh1)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	DrawLevelCh1
	rcall	MemToLCD
	mov	temp, keys_code
	clr	keys_code
	sbrc	temp, KEY_ENTER
	ldi	scrmode, MSHOWLEVELCH2
	ret
;--- show 2-d ch stereo level indicator
ShowLevelCh2:
	ldi	zl, low(2 * ScrShowLevelCh2)
	ldi	zh, high(2 * ScrShowLevelCh2)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	DrawLevelCh2
	rcall	MemToLCD
	mov	temp, keys_code
	clr	keys_code
	sbrc	temp, KEY_ENTER
	ldi	scrmode, MSHOWLEVELCH3
	ret
;--- show 3-d ch stereo level indicator
ShowLevelCh3:
	ldi	zl, low(2 * ScrShowLevelCh3)
	ldi	zh, high(2 * ScrShowLevelCh3)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	DrawLevelCh3
	rcall	MemToLCD
	mov	temp, keys_code
	clr	keys_code
	sbrc	temp, KEY_ENTER
	ldi	scrmode, MSHOWLEVELCH4
	ret
;--- show 4-d ch stereo level indicator
ShowLevelCh4:
	ldi	zl, low(2 * ScrShowLevelCh4)
	ldi	zh, high(2 * ScrShowLevelCh4)
	rcall	CopyScrTemplate; destroy tmp1
	rcall	DrawLevelCh4
	rcall	MemToLCD
	mov	temp, keys_code
	clr	keys_code
	sbrs	temp, KEY_ENTER
	ret
;--
	ldi	zl, low(2 * CharLevels)
	ldi	zh, high(2 * CharLevels)
	rcall WriteCharacters
	ldi	scrmode, MSHOWLEVELS
	ret
;****************
;- turn off cursor
TurnOffCursor:
	ldi	tmp1, 0x0c
;****************
;-     ,  -  tmp1,  temp
CommandToLCD:
	cbi	LCDCMDPORT, RSOUT
	ldi	temp, (2 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks

	out	LCDDATAPORT, tmp1
	ldi	temp, (2 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks

	sbi	LCDCMDPORT, EOUT
	ldi	temp, (4 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks
	cbi	LCDCMDPORT, EOUT
	rjmp	Wait_100mks
;****************
; show levels in the channels like bars
; Y, Z, tmp1, tmp2, barcode
DrawBarCh1:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL1)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL1)
	ldi	adc_channel, 0 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL1 + 1)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL1 + 1)
	ldi	adc_channel, 1 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ret
;--
DrawBarCh2:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL2)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL2)
	ldi	adc_channel, 2 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL2 + 1)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL2 + 1)
	ldi	adc_channel, 3 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ret
;--
DrawBarCh3:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL3)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL3)
	ldi	adc_channel, 4 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL3 + 1)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL3 + 1)
	ldi	adc_channel, 5 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ret
;--
DrawBarCh4:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL4)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL4)
	ldi	adc_channel, 6 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL4 + 1)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL4 + 1)
	ldi	adc_channel, 7 * LVLTBL_SIZE
	rcall	DrawVerticalBars
	ret
;****************
; calculate level.and draw vertical bars
;yl.yh = ram address of bar
;barcode = display char code of bar
;-51,(-48,-45),-42,(-39),-36,(-33),-30,-27,-24,-21,-18,-15,-12,-9,-6,-3,0
DrawVerticalBars:
	ldi	barcode, 0
;-- calculate bars of levels
	ldi	zl, low(RLEVEL_TABLE + AVERAGE_SHIFT)
	ldi	zh, high(RLEVEL_TABLE + AVERAGE_SHIFT)
	add	zl, adc_channel
	adc	zh, zero
	ld	adc_channel, z+
	ld	zh, z
	mov	zl, adc_channel
; -51 dB,    2
	ldi	tmp1, low(2)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -48 dB,    3, 4
;	ldi	tmp1, low(3)
;	mov	tmp2, zero
;	rcall	BarCompareLevel
; -45 dB,    4, 6
;	ldi	tmp1, low(4)
;	mov	tmp2, zero
;	rcall	BarCompareLevel
; -42 dB,    6, 8
	ldi	tmp1, low(6)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -39 dB,   8, 11
;	ldi	tmp1, low(8)
;	mov	tmp2, zero
;	rcall	BarCompareLevel
; -36 dB,   11, 16
	ldi	tmp1, low(11)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -33 dB,   16, 23
;	ldi	tmp1, low(16)
;	mov	tmp2, zero
;	rcall	BarCompareLevel
; -30 dB,   23, 32
	ldi	tmp1, low(23)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -27 dB,   32, 46
	ldi	tmp1, low(32)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -24 dB,   46, 64
	ldi	tmp1, low(46)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -21 dB,   64, 91
	ldi	tmp1, low(64)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -18 dB,  91, 128
	ldi	tmp1, low(91)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -15 dB,  129, 182
	ldi	tmp1, low(129)
	mov	tmp2, zero
	rcall	BarCompareLevel
; -12 dB,  182, 256
	ldi	tmp1, low(182)
	ldi	tmp2, high(182)
	rcall	BarCompareLevel
;  -9 dB,  256, 363
	ldi	tmp1, low(256)
	ldi	tmp2, high(256)
	rcall	BarCompareLevel
;  -6 dB,  362, 512
	ldi	tmp1, low(362)
	ldi	tmp2, high(362)
	rcall	BarCompareLevel
;  -3 dB,  512, 724
	ldi	tmp1, low(512)
	ldi	tmp2, high(512)
	rcall	BarCompareLevel
;   0 dB, 723, 1023
	ldi	tmp1, low(723)
	ldi	tmp2, high(723)
	rcall	BarCompareLevel
;
	ldi	tmp2, 7
	mov	tmp1, barcode
	sub	tmp1, tmp2
	brcc	DVB_high
	mov	tmp2, barcode
	mov	tmp1, zero
DVB_high:
	st	y, tmp1
	adiw	yl, LCD_STR_LENGTH
	st	y, tmp2
	ret
;****************
; compare level for vertical bar
; zl.zh = input level
; tmp1.tmp2 = treshold
; yl.yh = ram address
BarCompareLevel:
	cp	zl, tmp1
	cpc	zh, tmp2
	sbci barcode, -1
	ret
;****************
; show levels in the channels
; Y, Z, tmp1, tmp2, barcode
DrawLevelCh1:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	barcode, LCH_CHRCODE
	ldi	adc_channel, 0 * LVLTBL_SIZE
	rcall	DrawBars
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	barcode, RCH_CHRCODE
	ldi	adc_channel, 1 * LVLTBL_SIZE
	rcall	DrawBars
	ret
;--
DrawLevelCh2:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	barcode, LCH_CHRCODE
	ldi	adc_channel, 2 * LVLTBL_SIZE
	rcall	DrawBars
	ldi	yl, low(LCDbuffSTR2 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR2 + SHIFT_BARCHANNEL)
	ldi	barcode, RCH_CHRCODE
	ldi	adc_channel, 3 * LVLTBL_SIZE
	rcall	DrawBars
	ret
;--
DrawLevelCh3:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	barcode, LCH_CHRCODE
	ldi	adc_channel, 4 * LVLTBL_SIZE
	rcall	DrawBars
	ldi	yl, low(LCDbuffSTR2 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR2 + SHIFT_BARCHANNEL)
	ldi	barcode, RCH_CHRCODE
	ldi	adc_channel, 5 * LVLTBL_SIZE
	rcall	DrawBars
	ret
;--
DrawLevelCh4:
	ldi	yl, low(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR1 + SHIFT_BARCHANNEL)
	ldi	barcode, LCH_CHRCODE
	ldi	adc_channel, 6 * LVLTBL_SIZE
	rcall	DrawBars
	ldi	yl, low(LCDbuffSTR2 + SHIFT_BARCHANNEL)
	ldi	yh, high(LCDbuffSTR2 + SHIFT_BARCHANNEL)
	ldi	barcode, RCH_CHRCODE
	ldi	adc_channel, 7 * LVLTBL_SIZE
	rcall	DrawBars
	ret
;****************
; calculate bars.and draw
;yl.yh = ram address of bar
;barcode = display char code of bar
;-51,(-48),-45,(-42),-39,(-36),-33,(-30),-27,(-24),-21,-18,-15,-12,-9,-6,-3,0
DrawBars:
;-- calculate bars of levels
	ldi	zl, low(RLEVEL_TABLE + AVERAGE_SHIFT)
	ldi	zh, high(RLEVEL_TABLE + AVERAGE_SHIFT)
	add	zl, adc_channel
	adc	zh, zero
	ld	adc_channel, z+
	ld	zh, z
	mov	zl, adc_channel
; -51 dB,    2
	ldi	tmp1, low(2)
	mov	tmp2, zero
	rcall	CompareLevel
; -48 dB,    3, 4
.IF WH24
	ldi	tmp1, low(3)
	mov	tmp2, zero
	rcall	CompareLevel
.ENDIF
; -45 dB,    4, 6
	ldi	tmp1, low(4)
	mov	tmp2, zero
	rcall	CompareLevel
; -42 dB,    6, 8
.IF WH24
	ldi	tmp1, low(6)
	mov	tmp2, zero
	rcall	CompareLevel
.ENDIF
; -39 dB,   8, 11
	ldi	tmp1, low(8)
	mov	tmp2, zero
	rcall	CompareLevel
; -36 dB,   11, 16
.IF WH24
	ldi	tmp1, low(11)
	mov	tmp2, zero
	rcall	CompareLevel
.ENDIF
; -33 dB,   16, 23
	ldi	tmp1, low(16)
	mov	tmp2, zero
	rcall	CompareLevel
; -30 dB,   23, 32
.IF WH24
	ldi	tmp1, low(23)
	mov	tmp2, zero
	rcall	CompareLevel
.ENDIF
; -27 dB,   32, 46
	ldi	tmp1, low(32)
	mov	tmp2, zero
	rcall	CompareLevel
; -24 dB,   46, 64
.IF WH24
	ldi	tmp1, low(46)
	mov	tmp2, zero
	rcall	CompareLevel
.ENDIF
; -21 dB,   64, 91
	ldi	tmp1, low(64)
	mov	tmp2, zero
	rcall	CompareLevel
; -18 dB,  91, 128
	ldi	tmp1, low(91)
	mov	tmp2, zero
	rcall	CompareLevel
; -15 dB,  129, 182
	ldi	tmp1, low(129)
	mov	tmp2, zero
	rcall	CompareLevel
; -12 dB,  182, 256
	ldi	tmp1, low(182)
	ldi	tmp2, high(182)
	rcall	CompareLevel
;  -9 dB,  256, 363
	ldi	tmp1, low(256)
	ldi	tmp2, high(256)
	rcall	CompareLevel
;  -6 dB,  362, 512
	ldi	tmp1, low(362)
	ldi	tmp2, high(362)
	rcall	CompareLevel
;  -3 dB,  512, 724
	ldi	tmp1, low(512)
	ldi	tmp2, high(512)
	rcall	CompareLevel
;   0 dB, 723, 1023
	ldi	tmp1, low(723)
	ldi	tmp2, high(723)
	rcall	CompareLevel
	ret
;****************
; compare level
; zl.zh = input level
; tmp1.tmp2 = treshold
; yl.yh = ram address
CompareLevel:
	cp	zl, tmp1
	cpc	zh, tmp2
	sbc	tmp2, tmp2
	com	tmp2
	and	tmp2, barcode
	ld	tmp1, y
	or	tmp2, tmp1
	st	y+, tmp2
	ret
;****************
; initialization
Init:
; clear memory
	ldi	xl, low(RLEVEL_TABLE)
	ldi	xh, high(RLEVEL_TABLE)
	ldi	temp, QUANTITY_CHANNEL * LVLTBL_SIZE
init_loop:
	st	x+, zero
	dec	temp
	brne	init_loop
; read parameters from eeprom
; load gain 1
	ldi	temp, FM_GAININ_1
	rcall	EEPROM_read
	sts	RGAININ_1, temp
; load gain 2
	ldi	temp, FM_GAININ_2
	rcall	EEPROM_read
	sts	RGAININ_2, temp
; control
	lds	temp, RGAININ_1
	cpi	temp, SCALE_QUANTITY
	brcs	GainControl1_OK
	ldi	temp, SCALE_QUANTITY - 1
GainControl1_OK:
	sts	RGAININ_1, temp
	ldi	zl, low(2*LevelScale)
	ldi	zh, high(2*LevelScale)
	lsl	temp
	add	zl, temp
	adc	zh, zero
	lpm	temp, Z+
	lpm	zh, Z
	sts	RLEVELSCALE_1, temp
	sts	RLEVELSCALE_1 + 1, zh
;
	cpi	temp, SCALE_QUANTITY
	brcs	GainControl2_OK
	ldi	temp, SCALE_QUANTITY - 1
GainControl2_OK:
	sts	RGAININ_2, temp
	ldi	zl, low(2*LevelScale)
	ldi	zh, high(2*LevelScale)
	lsl	temp
	add	zl, temp
	adc	zh, zero
	lpm	temp, Z+
	lpm	zh, Z
	sts	RLEVELSCALE_2, temp
	sts	RLEVELSCALE_2 + 1, zh
; load mode
	ldi	temp, FM_MODE
	rcall	EEPROM_read
	mov	mode, temp
	ret
;****************
; calculate index of level
; X - 10 bit level
CalculateLevelIndex:
	ldi	zl, low(2 * (SqrtLevelsTable + LOGSSQRT_QUANTITY) - 1)
	ldi	zh, high(2 * (SqrtLevelsTable + LOGSSQRT_QUANTITY) - 1)
	ldi	temp, LOGSSQRT_QUANTITY
CLI_loop:
	lpm	yh, Z
	sbiw	zl, 1
	lpm	yl, Z
	sbiw	zl, 1
	cp	xl, yl
	cpc	xh, yh
	dec	temp
	brcc	CLI_ret
	brne	CLI_loop
CLI_ret:
	ret
;****************
; copy memory off screen buffer to lcd, use: temp, tmp1, tmp2, zl, zh
MemToLCD:
;    		
	ldi		tmp1, 0x80	;    DDRAM=0
	rcall	CommandToLCD
	ldi		ZL, low(LCDbuff)
	ldi		ZH, high(LCDbuff)
	ldi		tmp2, LCD_STR_LENGTH
MemToLCD_str1:
	ld		tmp1, Z+
	rcall	DataToLCD
	dec		tmp2
	brne	MemToLCD_str1
;    
	ldi		tmp1, 0xc0	;    DDRAM=40h
	rcall	CommandToLCD
	ldi		tmp2, LCD_STR_LENGTH
MemToLCD_str2:
	ld		tmp1, Z+
	rcall	DataToLCD
	dec		tmp2
	brne	MemToLCD_str2
	ret
;****************
; divide 8 / 8 unsigned
;yl		;remainder
;temp		;result
;temp		;dividend
;tmp1		;divisor
;yh		;loop counter
Div8u:
	sub	yl, yl	;clear remainder and carry
	ldi	yh, 9	;init loop counter
Div8u_1:
	rol	temp		;shift left dividend
	dec	yh		;decrement counter
	brne	Div8u_2		;if done
	ret			;    return
;--
Div8u_2:
	rol	yl		;shift dividend into remainder
	sub	yl, tmp1	;remainder = remainder - divisor
	brcc	Div8u_3		;if result negative
	add	yl, tmp1	;    restore remainder
	clc			;    clear carry to be shifted into result
	rjmp	Div8u_1		;else
Div8u_3:
	sec			;    set carry to be shifted into result
	rjmp	Div8u_1
;****************
; print dB
; Y - ram address string, temp - index of level
; destroy Z
PrintDBU:
	ldi	zl, low(2 * SqrtLogsTable)
	ldi	zh, high(2 * SqrtLogsTable)
	add	zl, temp
	adc	zh, zero
	lpm	temp, Z
;	neg	temp
	rcall	PrintUByte; destroy tmp1
	ret
;****************
; convert unsigned byte to string and print
; Y - ram address string, temp - unsigned byte
; destroy tmp1
PrintUByte:
	ser	tmp1
Div100_loop:
	subi	temp, 100
	inc	tmp1
	brcc	Div100_loop
	subi	temp, -100
	subi	tmp1, -'0'
	st	Y+, tmp1
;
	ser	tmp1
Div10_loop:
	subi	temp, 10
	inc	tmp1
	brcc	Div10_loop
	subi	temp, -10
	subi	tmp1, -'0'
	st	Y+, tmp1
	subi	temp, -'0'
	st	Y, temp
	ret
;****************
;   z -    , y -   
PrintString:
	lpm	temp, z+
	tst	temp
	breq	PrintStringRet
	st	y+, temp
	rjmp	PrintString
PrintStringRet:
	ret
;****************
; temp - address, temp - data
EEPROM_read:
; Wait for completion of previous write
	sbic	EECR, EEWE
	rjmp	EEPROM_read
; Set up address (temp) in address register
	out		EEARL, temp
; Start eeprom read by writing EERE
	sbi		EECR, EERE
; Read data from data register
	in		temp, EEDR
	ret
;******************
; write to eeprom
; temp - address, eeprom_data - data
EEPROM_write:
; Wait for completion of previous write
	sbic	EECR, EEWE
	rjmp	EEPROM_write
	out		EEARL, temp
	out		EEDR, eeprom_data
	in		eeprom_data, SREG
	cli
	sbi		EECR, EEMWE
	sbi		EECR, EEWE
	out		SREG, eeprom_data
	ret
;****************
; LCD init
LCDinit:
;-     
	cbi		LCDCMDPORT, RSOUT
	cbi		LCDCMDPORT, EOUT
; 15 
	ldi		tmp1, 150
	rcall	Wait_N
;		
	ldi		tmp1, 0x30
	rcall	CommandToLCD
;		
	rcall	Wait_N50

	ldi		tmp1, 0x30
	rcall	CommandToLCD

	ldi		tmp1, 0x30
	rcall	CommandToLCD

	ldi		tmp1, 0x38;0x3f 8  2  5x11, 0x38  5x8
	rcall	CommandToLCD
	ldi		tmp1, 0x08;    
	rcall	CommandToLCD
	ldi		tmp1, 0x01;     0   0,    
	rcall	CommandToLCD
;		
	rcall	Wait_N50

	ldi		tmp1, 0x06;     
	rcall	CommandToLCD
	ldi		tmp1, 0x3f
	rcall	CommandToLCD
	ldi		tmp1, 0x08
	rcall	CommandToLCD
	ldi		tmp1, 0x0c;    
	rcall	CommandToLCD
	ldi		tmp1, 0x06
	rcall	CommandToLCD
	ret
;****************
;  
;WriteCharacter:
;	ldi	zl, low(2 * Characters)
;	ldi	zh, high(2 * Characters)
WriteCharacters:
	ldi	yh, 8; character quantity
	ldi	tmp2, 0x40; cgram start address
WriteCharacter_1:
	ldi	yl, 8; symbol height
WC_loop:
	mov	tmp1, tmp2
	rcall	CommandToLCD
	lpm	tmp1, Z+
	rcall	DataToLCD
	inc	tmp2
	dec	yl
	brne	WC_loop
	dec	yh
	brne	WriteCharacter_1
	ret
;***************
; copy screen template to lcd ram buffer
; zl.zh = address of screen
; destroy: temp, tmp1, Y
CopyScrTemplate:
	ldi	yl, low(LCDbuff)
	ldi	yh, high(LCDbuff)
	ldi	temp, LCD_STR_LENGTH * 2
CopyScrTemplate_loop:
	lpm	tmp1, z+
	st	y+, tmp1
	dec	temp
	brne	CopyScrTemplate_loop
	ret
;****************
;-   N*100.  tmp1, temp
Wait_N50:
	ldi	tmp1, 50
Wait_N:
	rcall	Wait_100mks
	dec	tmp1
	brne	Wait_N
	ret
;****************
;-     ,  -  tmp1,  temp
DataToLCD:
	sbi	LCDCMDPORT, RSOUT
	ldi	temp, (2 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks

	out	LCDDATAPORT, tmp1
	ldi	temp, (2 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks
	sbi	LCDCMDPORT, EOUT
	ldi	temp, (4 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks
	cbi	LCDCMDPORT, EOUT
;----------------
; wait 100 mks
Wait_100mks:
	ldi	temp, (40 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks
	ldi	temp, (40 * QUARTZ / 1000000 - 9) / 3
	rcall	Wait_Nmks
	ldi	temp, (20 * QUARTZ / 1000000 - 9) / 3
;----------------
; wait N mks, temp - input data, 3...193.5mks@4MHz
; Nreg = (Nmks * QUARTZ / 1000000 - 9) / 3
Wait_Nmks:
	dec	temp
	brne	WAIT_Nmks
	ret
;****************
; dB table sqrt
.equ LOGSSQRT_QUANTITY	= 47
SqrtLevelsTable:
.dw  1,  2,  3,  4,  5,  6,  7,  8,  9, 10
.dw 11, 13, 14, 16, 18, 20, 23, 26, 29, 32
.dw 36, 41, 46, 51, 57, 64, 72, 81, 91, 102
.dw 115,128,144,162,182,204,229,257,288,323
.dw 363,407,456,512,575,645,726,730
;----------------
SqrtLogsTable:
; negative logs
.db 57, 51, 48, 45, 43, 42, 40, 39, 38, 37
.db 36, 35, 34, 33, 32, 31, 30, 29, 28, 27
.db 26, 25, 24, 23, 22, 21, 20, 19, 18, 17
.db 16, 15, 14, 13, 12, 11, 10,  9,  8,  7
.db  6,  5,  4,  3,  2,  1,  0,  0
;****************
; dB table, 50 numbers
.equ LOGS_QUANTITY	= 50
LevelsTable:
.dw	  1,  2,  3,  4,  5,  6,  7,  8,  9, 10
.dw	 11, 12, 14, 16, 18, 20, 22, 25, 28, 32
.dw	 36, 40, 45, 51, 57, 64, 72, 81, 91, 102
.dw	114,128,144,162,181,204,229,256,288,323
.dw	362,407,456,512,575,645,724,812,911,1023
;----------------
LogsTable:
; negative logs
;    1    2    3    4    5    6    7    8    9    10
.db	 60,  54,  50,  48,  46,  44,  43,  42,  41,  40
;    11   12   14   16   18   20   22   25   28   32
.db	 39,  38,  37,  36,  35,  34,  33,  32,  31,  30
;    36   40   45   51   57   64   72   81   91  102
.db	 29,  28,  27,  26,  25,  24,  23,  22,  21,  20
;   114  128  144  162  181  204  229  256  288  323
.db	 19,  18,  17,  16,  15,  14,  13,  12,  11,  10
;   362  407  456  512  575  645  724  812  911 1023
.db	 9,   8,   7,   6,   5,   4,   3,   2,   1,  0
;****************
; 0.5 dB scales
.equ SCALE_QUANTITY	= 16
LevelScale:
.dw   256,   271,   287,   304,   322,   341,   362,   383,   406,   430
.dw   455,   482,   511,   541,   573,   607;,   643,   681,   722,   764
;.dw   810,   858,   908,   962,  1019,  1080,  1144,  1211,  1283,  1359
;.dw  1440,  1525,  1615,  1711,  1812,  1920,  2033,  2154,  2282,  2417
;.dw  2560,  2712,  2872,  3043,  3223,  3414,  3616,  3830,  4057,  4298
;.dw  4552,  4822,  5108,  5411,  5731,  6071,  6430,  6811,  7215,  7643
;.dw  8095,  8575,  9083,  9621, 10192, 10795, 11435, 12113, 12830, 13591
;.dw 14396, 15249, 16153, 17110, 18123, 19197, 20335, 21540, 22816, 24168
;****************
; screen template
.IF WH24
ScrShowID:
.db	"       LEVEL METER      "
.db	"       (c)  V*D*V       "
;
ScrSensorUnplugged:
.db	255,255,255,255,"Sensor Unplugged",255,255,255,255
.db	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
;--
.equ SHIFT_BARCHANNEL1 = 4
.equ SHIFT_BARCHANNEL2 = 10
.equ SHIFT_BARCHANNEL3 = 16
.equ SHIFT_BARCHANNEL4 = 22
ScrShowLevels:
.db	"    II    II    II    II"
.db	" O1 II O2 II O3 II O4 II"
;Screen_01:
;.db	"                        "
;.db	"-60-50-40-30-20-10 0 Lev"
;Screen_Show:
;.db	"EDIT ================== "
;.db	"AUTO ================== "
.equ SHIFT_BARCHANNEL	= 4; 18 bars
ScrShowLevelCh1:
.db	"   ",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"  "
.db	"       CNANNEL 1        "
ScrShowLevelCh2:
.db	"   ",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"  "
.db	"       CNANNEL 2        "
ScrShowLevelCh3:
.db	"   ",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"  "
.db	"       CNANNEL 3        "
ScrShowLevelCh4:
.db	"   ",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"  "
.db	"       CNANNEL 4        "
;
  .ELSE;if wh24
ScrShowID:
.db	"   LEVEL METER  "
.db	"   (c)  V*D*V   "
;
ScrSensorUnplugged:
.db	"Sensor Unplugged"
.db	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
;--
.equ SHIFT_BARCHANNEL1 = 2
.equ SHIFT_BARCHANNEL2 = 6
.equ SHIFT_BARCHANNEL3 = 10
.equ SHIFT_BARCHANNEL4 = 14
ScrShowLevels:
.db	"  II  II  II  II"
.db	"O1IIO2IIO3IIO4II"
;Screen_01:
;.db	"                        "
;.db	"-60-50-40-30-20-10 0 Lev"
;Screen_Show:
;.db	"EDIT ================== "
;.db	"AUTO ================== "
.equ SHIFT_BARCHANNEL	= 3; 13 bars
ScrShowLevelCh1:
.db	"O1",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0
.db	"   CHANNEL 1    "
ScrShowLevelCh2:
.db	"O2",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0
.db	"   CHANNEL 2    "
ScrShowLevelCh3:
.db	"O3",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0
.db	"   CHANNEL 3    "
ScrShowLevelCh4:
.db	"O4",LR_CHRCODE,0,0,0,0,0,0,0,0,0,0,0,0,0
.db	"   CHANNEL 4    "
.ENDIF;if wh24
;----
.equ LR_CHRCODE	= 0x07
.equ LCH_CHRCODE	= 0x01
.equ RCH_CHRCODE	= 0x02
Characters:
; ,0x00
.db	0,0,0,0,0,0,0,0
;-,0x01
.db	0,255,255,0,0,0,0,0
;_,0x02
.db	0,0,0,0,255,255,0,0
;=,0x03
.db	0,255,255,0,255,255,0,0
; ,0x04
.db	4,4,14,14,31,31,0,0
;^,0x05
.db	4,14,21,4,4,7,0,0
;<-,0x06
.db	1,5,9,31,8,4,0,0
;LR,0x07
.db	4,4,7,0,7,6,5,0
;***************************
CharLevels:
; ,0x00
.db	0,0,0,0,0,0,0,0
;-,0x01
.db	0,0,0,0,0,0,255,0
;_,0x02
.db	0,0,0,0,0,255,255,0
;_,0x03
.db	0,0,0,0,255,255,255,0
;_,0x04
.db	0,0,0,255,255,255,255,0
;_,0x05
.db	0,0,255,255,255,255,255,0
;_,0x06
.db	0,255,255,255,255,255,255,0
;_,0x07
.db	255,255,255,255,255,255,255,0
